@@ -1,6 +1,6 @@
 /* tc-msp430.c -- Assembler code for the Texas Instruments MSP430
 
-  Copyright (C) 2002, 2003, 2004 Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
   Contributed by Dmitry Diky <diwil@mail.ru>
 
   This file is part of GAS, the GNU Assembler.
@@ -17,8 +17,8 @@
 
   You should have received a copy of the GNU General Public License
   along with GAS; see the file COPYING.  If not, write to
-  the Free Software Foundation, 59 Temple Place - Suite 330,
-  Boston, MA 02111-1307, USA.  */
+  the Free Software Foundation, 51 Franklin Street - Fifth Floor,
+  Boston, MA 02110-1301, USA.  */
 
 #include <stdio.h>
 #include <string.h>
@@ -1295,22 +1295,6 @@ msp430_srcoperand (struct msp430_operand_s * op,
   /* Symbolic mode 'mov a, b' == 'mov x(pc), y(pc)'.  */
   do
     {
-#if 0  /* Allow expression in operand like 'a+123*(1|2)'.  */
-      char *t = l;
-
-      __tl = l;
-
-      while (*t)
-	{
-	  /* alpha/number    underline     dot for labels.  */
-	  if (! ISALNUM (*t) && *t != '_' && *t != '.')
-	    {
-	      as_bad (_("unknown operand %s"), l);
-	      return 1;
-	    }
-	  t++;
-	}
-#endif
       op->mode = OP_EXP;
       op->reg = 0;		/* PC relative... be careful.  */
       op->am = 1;
@@ -1834,7 +1818,7 @@ md_pcrel_from_section (fixS * fixp, segT sec)
    value in the object file.  */
 
 void
-md_apply_fix3 (fixS * fixp, valueT * valuep, segT seg)
+md_apply_fix (fixS * fixp, valueT * valuep, segT seg)
 {
   unsigned char * where;
   unsigned long insn;
@@ -1851,7 +1835,19 @@ md_apply_fix3 (fixS * fixp, valueT * valuep, segT seg)
 
       if (fixp->fx_addsy && (s == seg || s == absolute_section))
 	{
-	  value = S_GET_VALUE (fixp->fx_addsy) + *valuep;
+	  /* FIXME: We can appear here only in case if we perform a pc
+	     relative jump to the label which is i) global, ii) locally
+	     defined or this is a jump to an absolute symbol.
+	     If this is an absolute symbol -- everything is OK.
+	     If this is a global label, we've got a symbol value defined
+	     twice:
+               1. S_GET_VALUE (fixp->fx_addsy) will contain a symbol offset
+	          from this section start
+               2. *valuep will contain the real offset from jump insn to the
+	          label
+	     So, the result of S_GET_VALUE (fixp->fx_addsy) + (* valuep);
+	     will be incorrect. Therefore remove s_get_value.  */
+	  value = /* S_GET_VALUE (fixp->fx_addsy) + */ * valuep;
 	  fixp->fx_done = 1;
 	}
       else
@@ -1891,7 +1887,7 @@ md_apply_fix3 (fixS * fixp, valueT * valuep, segT seg)
       /* Fetch the instruction, insert the fully resolved operand
 	 value, and stuff the instruction back again.  */
 
-      where = fixp->fx_frag->fr_literal + fixp->fx_where;
+      where = (unsigned char *) fixp->fx_frag->fr_literal + fixp->fx_where;
 
       insn = bfd_getl16 (where);
 
