@@ -1,6 +1,6 @@
 /* output-file.c -  Deal with the output file
-   Copyright 1987, 1990, 1991, 1992, 1993, 1994, 1996, 1998, 1999, 2001
-   Free Software Foundation, Inc.
+   Copyright 1987, 1990, 1991, 1992, 1993, 1994, 1996, 1998, 1999, 2001,
+   2003, 2004 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GAS; see the file COPYING.  If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   the Free Software Foundation, 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 #include <stdio.h>
 
@@ -41,15 +41,17 @@
 bfd *stdoutput;
 
 void
-output_file_create (name)
-     char *name;
+output_file_create (char *name)
 {
   if (name[0] == '-' && name[1] == '\0')
-    as_fatal (_("Can't open a bfd on stdout %s "), name);
+    as_fatal (_("can't open a bfd on stdout %s"), name);
 
   else if (!(stdoutput = bfd_openw (name, TARGET_FORMAT)))
     {
-      as_perror (_("FATAL: Can't create %s"), name);
+      if (bfd_get_error () == bfd_error_invalid_target)
+	as_perror (_("Selected target format '%s' unknown"), TARGET_FORMAT);
+      else
+	as_perror (_("FATAL: can't create %s"), name);
       exit (EXIT_FAILURE);
     }
 
@@ -62,22 +64,21 @@ output_file_create (name)
 }
 
 void
-output_file_close (filename)
-     char *filename;
+output_file_close (char *filename)
 {
 #ifdef BFD_ASSEMBLER
   /* Close the bfd.  */
   if (bfd_close (stdoutput) == 0)
     {
       bfd_perror (filename);
-      as_perror (_("FATAL: Can't close %s\n"), filename);
+      as_perror (_("FATAL: can't close %s\n"), filename);
       exit (EXIT_FAILURE);
     }
 #else
   /* Close the bfd without getting bfd to write out anything by itself.  */
   if (bfd_close_all_done (stdoutput) == 0)
     {
-      as_perror (_("FATAL: Can't close %s\n"), filename);
+      as_perror (_("FATAL: can't close %s\n"), filename);
       exit (EXIT_FAILURE);
     }
 #endif
@@ -86,10 +87,9 @@ output_file_close (filename)
 
 #ifndef BFD_ASSEMBLER
 void
-output_file_append (where, length, filename)
-     char *where ATTRIBUTE_UNUSED;
-     long length ATTRIBUTE_UNUSED;
-     char *filename ATTRIBUTE_UNUSED;
+output_file_append (char *where ATTRIBUTE_UNUSED,
+		    long length ATTRIBUTE_UNUSED,
+		    char *filename ATTRIBUTE_UNUSED)
 {
   abort ();
 }
@@ -100,8 +100,7 @@ output_file_append (where, length, filename)
 static FILE *stdoutput;
 
 void
-output_file_create (name)
-     char *name;
+output_file_create (char *name)
 {
   if (name[0] == '-' && name[1] == '\0')
     {
@@ -110,26 +109,25 @@ output_file_create (name)
     }
 
   stdoutput = fopen (name, FOPEN_WB);
-
-  /* Some systems don't grok "b" in fopen modes.  */
-  /* XXX - is this still necessary now that we use FOPEN_WB ?  */
-  if (stdoutput == NULL)
-    stdoutput = fopen (name, FOPEN_W);
-
   if (stdoutput == NULL)
     {
-      as_perror (_("FATAL: Can't create %s"), name);
+#ifdef BFD_ASSEMBLER
+      bfd_set_error (bfd_error_system_call);
+#endif
+      as_perror (_("FATAL: can't create %s"), name);
       exit (EXIT_FAILURE);
     }
 }
 
 void
-output_file_close (filename)
-     char *filename;
+output_file_close (char *filename)
 {
   if (EOF == fclose (stdoutput))
     {
-      as_perror (_("FATAL: Can't close %s"), filename);
+#ifdef BFD_ASSEMBLER
+      bfd_set_error (bfd_error_system_call);
+#endif
+      as_perror (_("FATAL: can't close %s"), filename);
       exit (EXIT_FAILURE);
     }
 
@@ -138,20 +136,19 @@ output_file_close (filename)
 }
 
 void
-output_file_append (where, length, filename)
-     char * where;
-     long   length;
-     char * filename;
+output_file_append (char * where, long length, char * filename)
 {
   for (; length; length--, where++)
     {
       (void) putc (*where, stdoutput);
 
       if (ferror (stdoutput))
-	/* if ( EOF == (putc( *where, stdoutput )) ) */
 	{
+#ifdef BFD_ASSEMBLER
+	  bfd_set_error (bfd_error_system_call);
+#endif
 	  as_perror (_("Failed to emit an object byte"), filename);
-	  as_fatal (_("Can't continue"));
+	  as_fatal (_("can't continue"));
 	}
     }
 }
