@@ -1,5 +1,5 @@
 /* Opcode table for the ARC.
-   Copyright 1994, 1995, 1997, 1998, 2000, 2001
+   Copyright 1994, 1995, 1997, 1998, 2000, 2001, 2002, 2004
    Free Software Foundation, Inc.
    Contributed by Doug Evans (dje@cygnus.com).
 
@@ -15,7 +15,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "sysdep.h"
 #include <stdio.h>
@@ -66,6 +66,9 @@ enum operand {OP_NONE,OP_REG,OP_SHIMM,OP_LIMM};
 
 enum operand ls_operand[OPERANDS];
 
+struct arc_opcode *arc_ext_opcodes;
+struct arc_ext_operand_value *arc_ext_operands;
+
 #define LS_VALUE  0
 #define LS_DEST   0
 #define LS_BASE   1
@@ -935,6 +938,7 @@ insert_base (insn, operand, mods, reg, value, errmsg)
       shimm_p = 1;
       shimm = value;
       ls_operand[LS_BASE] = OP_SHIMM;
+      ls_operand[LS_OFFSET] = OP_SHIMM;
     }
   else
     {
