@@ -1,5 +1,6 @@
 /* ia64-gen.c -- Generate a shrunk set of opcode tables
-   Copyright 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright 1999, 2000, 2001, 2002, 2004, 2005
+   Free Software Foundation, Inc.
    Written by Bob Manson, Cygnus Solutions, <manson@cygnus.com>
 
    This file is part of GDB, GAS, and the GNU binutils.
@@ -16,8 +17,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this file; see the file COPYING.  If not, write to the
-   Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Free Software Foundation, 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 /* While the ia64-opc-* set of opcode tables are easy to maintain,
    they waste a tremendous amount of space.  ia64-gen rearranges the
@@ -56,6 +57,7 @@
 const char * program_name = NULL;
 int debug = 0;
 
+#define NELEMS(a) (sizeof (a) / sizeof ((a)[0]))
 #define tmalloc(X) (X *) xmalloc (sizeof (X))
 
 /* The main opcode table entry.  Each entry is a unique combination of
@@ -1061,10 +1063,11 @@ in_iclass (struct ia64_opcode *idesc, struct iclass *ic,
                   && (idesc->name[len] == '\0' 
                       || idesc->name[len] == '.'));
 
-      /* All break and nop variations must match exactly.  */
+      /* All break, nop, and hint variations must match exactly.  */
       if (resolved &&
           (strcmp (ic->name, "break") == 0
-           || strcmp (ic->name, "nop") == 0))
+           || strcmp (ic->name, "nop") == 0
+	   || strcmp (ic->name, "hint") == 0))
         resolved = strcmp (ic->name, idesc->name) == 0;
 
       /* Assume restrictions in the FORMAT/FIELD negate resolution,
@@ -1133,7 +1136,13 @@ in_iclass (struct ia64_opcode *idesc, struct iclass *ic,
           else if (strcmp (ic->name, "invala") == 0)
             resolved = strcmp (idesc->name, ic->name) == 0;
 	  else if (strncmp (idesc->name, "st", 2) == 0
-		   && strstr (format, "M5") != NULL)
+		   && (strstr (format, "M5") != NULL
+		       || strstr (format, "M10") != NULL))
+	    resolved = idesc->flags & IA64_OPCODE_POSTINC;
+	  else if (strncmp (idesc->name, "ld", 2) == 0
+		   && (strstr (format, "M2 M3") != NULL
+		       || strstr (format, "M12") != NULL
+		       || strstr (format, "M7 M8") != NULL))
 	    resolved = idesc->flags & IA64_OPCODE_POSTINC;
           else
             resolved = 0;
@@ -1257,6 +1266,22 @@ lookup_regindex (const char *name, int specifier)
         return 18;
       else if (strstr (name, "[RNAT]"))
         return 19;
+      else if (strstr (name, "[FCR]"))
+        return 21;
+      else if (strstr (name, "[EFLAG]"))
+        return 24;
+      else if (strstr (name, "[CSD]"))
+        return 25;
+      else if (strstr (name, "[SSD]"))
+        return 26;
+      else if (strstr (name, "[CFLG]"))
+        return 27;
+      else if (strstr (name, "[FSR]"))
+        return 28;
+      else if (strstr (name, "[FIR]"))
+        return 29;
+      else if (strstr (name, "[FDR]"))
+        return 30;
       else if (strstr (name, "[CCV]"))
         return 32;
       else if (strstr (name, "[ITC]"))
@@ -1546,7 +1571,7 @@ print_dependency_table ()
   for (i=0;i < dlistlen;i++)
     {
       int len = 2;
-      printf ("static const short dep%d[] = {\n  ", i);
+      printf ("static const unsigned short dep%d[] = {\n  ", i);
       for (j=0;j < dlists[i]->len; j++)
         {
           len += printf ("%d, ", dlists[i]->deps[j]);
@@ -1889,7 +1914,7 @@ gen_dis_table (ent)
   if ((needed_bytes + insn_list_len) > tot_insn_list_len)
     {
       tot_insn_list_len += 256;
-      insn_list = (char *) xrealloc (insn_list, tot_insn_list_len);
+      insn_list = (unsigned char *) xrealloc (insn_list, tot_insn_list_len);
     }
   our_offset = insn_list_len;
   insn_list_len += needed_bytes;
@@ -2040,7 +2065,7 @@ gen_dis_table (ent)
 	  /* Store the address of the entry being branched to.  */
 	  while (currbits >= 0)
 	    {
-	      char *byte = insn_list + our_offset + bitsused / 8;
+	      unsigned char *byte = insn_list + our_offset + bitsused / 8;
 
 	      if (idest & (1 << currbits))
 		*byte |= (1 << (7 - (bitsused % 8)));
@@ -2702,7 +2727,26 @@ shrink (table)
   int curr_opcode;
 
   for (curr_opcode = 0; table[curr_opcode].name != NULL; curr_opcode++)
-    add_opcode_entry (table + curr_opcode);
+    {
+      add_opcode_entry (table + curr_opcode);
+      if (table[curr_opcode].num_outputs == 2
+	  && ((table[curr_opcode].operands[0] == IA64_OPND_P1
+	       && table[curr_opcode].operands[1] == IA64_OPND_P2)
+	      || (table[curr_opcode].operands[0] == IA64_OPND_P2
+		  && table[curr_opcode].operands[1] == IA64_OPND_P1)))
+	{
+	  struct ia64_opcode *alias = tmalloc(struct ia64_opcode);
+	  unsigned i;
+
+	  *alias = table[curr_opcode];
+	  for (i = 2; i < NELEMS (alias->operands); ++i)
+	    alias->operands[i - 1] = alias->operands[i];
+	  alias->operands[NELEMS (alias->operands) - 1] = IA64_OPND_NIL;
+	  --alias->num_outputs;
+	  alias->flags |= PSEUDO;
+	  add_opcode_entry (alias);
+	}
+    }
 }
 
 
